#ifndef AlienCommandList_h
#define AlienCommandList_h

// $Id: AlienCommandList.h,v 1.8 2014/01/28 16:40:41 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "AlienCommand.h"

#include <QVector>

class AlienCommandList: public Counter
{

    public:

    //! constructor
    AlienCommandList( void ):
        Counter( "AlienCommandList" )
    {}

    //! destructor
    virtual ~AlienCommandList( void )
    {}

    //! command set (read only)
    const SortedAlienCommand::Set& commands( void ) const
    { return set_; }

    //! size
    unsigned int size( void ) const
    { return set_.size(); }

    //! return true if empty
    bool isEmpty( void ) const
    { return set_.isEmpty(); }

    //! true if has pending commands (beside the running one)
    bool hasPendingCommands( void )
    { return size() > 1; }

    //! return front command
    const AlienCommand& front( void ) const;

    //! append
    /*! returns true if appended */
    bool append( const AlienCommand& );

    //! pop front
    bool removeFirst( void );

    //! abort a given command from list. Return true if aborted
    /*! note: the "front" command is never aborted */
    bool abort( const AlienCommand& );

    private:

    typedef QVector<AlienCommand::List> PriorityList;

    //! used to find first non empty
    class NonEmptyFTor
    {
        public:

        bool operator() (const AlienCommand::List& list )
        { return !list.isEmpty(); }

    };

    //! list of commands
    PriorityList list_;

    //! set of commands to be kept in sync
    SortedAlienCommand::Set set_;

    //! front command
    AlienCommand front_;

};

#endif
