#ifndef AlienCommandModel_h
#define AlienCommandModel_h

// $Id: AlienCommandModel.h,v 1.9 2014/02/18 15:15:56 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "ListModel.h"
#include "AlienCommand.h"

#include <QHash>

//! Job model. Stores job information for display in lists
class AlienCommandModel : public ListModel<AlienCommand>, public Counter
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor
    AlienCommandModel(QObject *parent = 0);

    //! destructor
    virtual ~AlienCommandModel()
    {}

    //! column type enumeration
    enum ColumnType
    {
        Id,
        Priority,
        Name,
        Arguments,
        nColumns
    };

    //!@name methods reimplemented from base class
    //@{

    //! data
    virtual QVariant data(const QModelIndex &index, int role) const;

    //! header data
    virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;

    //! number of columns for a given index
    // virtual int columnCount(const QModelIndex &parent = QModelIndex()) const
    virtual int columnCount(const QModelIndex & = QModelIndex()) const
    { return nColumns; }

    //@}

    public Q_SLOTS:

    //! add command
    virtual void add( const AlienCommand& command )
    { ListModel<AlienCommand>::add( command ); }

    //! add commands
    virtual void add( const AlienCommand::List& commands )
    { ListModel<AlienCommand>::add( commands ); }

    //! remove
    virtual void remove( const AlienCommand& command )
    { ListModel<AlienCommand>::remove( command ); }

    //! update configuration
    void updateConfiguration( void );

    protected:

    //! sort
    virtual void _sort( int column, Qt::SortOrder order = Qt::AscendingOrder )
    { std::sort( _get().begin(), _get().end(), SortFTor( column, order ) ); }

    private:

    //! used to sort IconCaches
    class SortFTor: public ItemModel::SortFTor
    {

        public:

        //! constructor
        SortFTor( const int& type, Qt::SortOrder order = Qt::AscendingOrder ):
            ItemModel::SortFTor( type, order )
        {}

        //! prediction
        bool operator() ( AlienCommand , AlienCommand ) const;

    };

    //! install pixmaps
    void _installIcons( void ) const;

    //! icon from command type
    static const QIcon& _icon( AlienCommand::Type );

    //! icon cache
    typedef QHash<AlienCommand::Type, QIcon> IconCache;

    static IconCache& _icons( void );

    //! list column names
    static const QString columnTitles_[nColumns];

};

#endif
