#ifndef JobSet_h
#define JobSet_h

// $Id: JobSet.h,v 1.7 2013/04/29 18:27:12 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "Job.h"
#include "JobModel.h"

#include <QDomDocument>
#include <QDomElement>
#include <QSet>

//! set of jobs
class JobSet: public QSet<Job>
{

    public:

    //! constructor
    JobSet():
        active_( false )
    {}

    //! constructor
    JobSet( const QSet<Job>& other ):
        QSet<Job>( other ),
        active_( false )
    {}

    //! construct from dom element
    JobSet( const QDomElement& );

    //! dom element
    QDomElement domElement( QDomDocument& ) const;

    //! owner
    QString owner( void ) const
    { return owner_; }

    //! owner
    void setOwner( const QString& owner )
    { owner_ = owner; }

    //! active
    bool isActive( void ) const
    { return active_; }

    //! active
    void setActive( bool value )
    { active_ = value; }

    //! assign children flags
    void checkChildren( void );

    //! convert to list
    JobModel::List toList( void ) const
    {
        JobModel::List out;
        foreach( const Job& job, *this )
        { out << job; }

        return out;
    }

    private:

    //! active
    bool active_;

    //! owner
    QString owner_;

};

#endif
