#ifndef MainWindow_h
#define MainWindow_h

// $Id: MainWindow.h,v 1.58 2014/03/21 11:50:54 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "AlienCommandModel.h"
#include "BaseMainWindow.h"
#include "BusyWidget.h"
#include "Counter.h"
#include "CustomToolBar.h"
#include "Debug.h"
#include "Job.h"
#include "AlienCommand.h"
#include "Key.h"
#include "UserSet.h"

#include <QCloseEvent>

class AlienInterface;
class JobManagerFrame;
class JobSet;
class LogDialog;
class ProgressStatusBar;
class SummaryFrame;
class UserSelectionFrame;
class AnimatedTabWidget;

/*!
\class MainWindow
\brief customized QVBox to recieve close event
*/
class MainWindow: public BaseMainWindow, public Base::Key
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor
    MainWindow( QWidget* parent = 0 );

    //! destructor
    virtual ~MainWindow( void );

    //! user
    UserSelectionFrame& userSelectionFrame( void ) const
    { return *userSelectionFrame_; }

    //! toolbar
    CustomToolBar& userToolBar( void )
    { return *userToolBar_; }

    //! toolbar
    CustomToolBar& toolBar( void )
    { return *toolbar_; }

    //! status bar
    ProgressStatusBar& statusBar( void ) const
    { return *statusBar_; }

    //! install frames
    void installFrames( void );

    //!@name actions
    //@{

    // connect
    QAction& connectAction( void ) const
    { return *connectAction_; }

    //! command list action
    QAction& commandListAction( void ) const
    { return *commandListAction_; }

    //! log action
    QAction& logAction( void ) const
    { return *logAction_; }

    //! log action
    QAction& toggleLogAction( void ) const
    { return *toggleLogAction_; }

    //! hide killed jobs
    QAction& hideKilledJobsAction( void ) const
    { return *hideKilledJobsAction_; }

    //! pause action
    QAction& pauseAction( void ) const
    { return *pauseAction_; }

    //! default user action
    QAction& defaultUserAction( void ) const
    { return *defaultUserAction_; }

    //! user list action
    QAction& userListAction( void ) const
    { return *userListAction_; }

    //! toggle summary
    QAction& toggleSummaryAction( void ) const
    { return *toggleSummaryAction_; }

    //! update
    QAction& updateAction( void ) const
    { return *updateAction_; }

    //! kill
    QAction& killAction( void ) const
    { return *killAction_; }

    //! add frame
    QAction& addFrameAction( void ) const
    { return *addFrameAction_; }

    //! remove current frame
    QAction& removeFrameAction( void ) const
    { return *removeFrameAction_; }

    //! resubmit
    QAction& resubmitAction( void ) const
    { return *resubmitAction_; }

    //! retrace
    QAction& traceAction( void ) const
    { return *traceAction_; }

    //! clear
    QAction& clearAction( void ) const
    { return *clearAction_; }

    //! abort
    QAction& abortAction( void ) const
    { return *abortAction_; }

    //! job dialogs
    QAction& closeJobDialogsAction( void ) const
    { return *closeJobDialogsAction_; }

    //@}

    Q_SIGNALS:

    //! request command
    // TODO: remove
    void commandRequested( const AlienCommand& );

    //! request commands
    // TODO: remove
    void commandsRequested( const AlienCommand::List& );

    //! user list ready
    void userListAvailable(QStringList);

    //! active users have changed
    void activeUsersChanged( UserSet );

    //! scratch file
    void scratchFileCreated( File );

    protected Q_SLOTS:

    //! check proxy
    void _checkProxy( void );

    //! connect
    void _connect( void );

    //! connection state changed
    void _connectionStateChanged( bool );

    //! connection lost
    void _lostConnection( const QString& );

    //! close job dialogs
    virtual void _closeJobInformationDialogs( void );

    //! request default user
    void _requestDefaultUser( void );

    //! request user list
    void _requestUserList( void );

    //! request job list
    void _requestJobList( void );

    //! store default user
    void _updateDefaultUser( QString );

    //! process user list
    void _processUserList( UserSet );

    //! process job list
    void _processMasterList( QString, const JobSet& );

    //! process job list
    void _processChildrenList( Job, const JobSet& );

     //! process completed command
    void _commandCompleted( const AlienCommand& );

    //! commands failed
    void _commandsFailed( const AlienCommand::List& );

    //! busy
    virtual void _busy( void )
    { busyWidget_->start(); }

    //! busy
    virtual void _idle( void )
    { busyWidget_->stop(); }

    //! update actions visibility
    void _updateActions( void );

    //! update clear action
    void _updateClearAction( void );

    //! update user
    void _updateUser( QString );

    //! update status bar
    void _updateStatusBar( QString );

    //! active frame changed
    void _activeFrameChanged( int );

    //! toggle pause
    void _togglePause( bool );

    //! toggle summary
    void _toggleSummary( bool );

    //! add frame
    void _addFrame( void );

    //! remove frame
    void _removeFrame( void );

    //! request children list
    void _requestChildrenList( const Job& );

    //! retrace jobs
    void _requestJobInformation( void );

    //! kill jobs
    void _kill( void );

    //! resubmit jobs
    void _resubmit( void );

    //! clear completed jobs
    void _clear( void );

    //! show command list
    void _showCommandList( void );

    //! show log
    void _showLog( void );

    //! toggle log
    void _toggleLog( bool );

    //! toggle hide killed jobs
    void _toggleHideKilledJobs( bool );

    //! log dialog hidden
    void _logDialogHidden( void );

    //! command list destroyed
    void _commandListDestroyed( void );

    protected:

    //! timer event
    void timerEvent( QTimerEvent* );

    //! close window event handler
    void closeEvent( QCloseEvent* );

    //! active frame
    bool _hasActiveFrame( void ) const
    { return (bool) activeFrame_; }

    //! active frame
    JobManagerFrame& _activeFrame( void ) const
    { return *activeFrame_; }

    //! true if frame is active
    bool _isActiveFrame( const JobManagerFrame& frame ) const
    { return &frame == activeFrame_; }

    //! find frame matching user
    /*! return 0 if none found */
    JobManagerFrame* _findFrame( const QString& user ) const;

    //! active frame
    bool _activeFrameIsDefault( void ) const;

    //! request job list
    void _requestJobList( QString );

    //! request job list
    void _requestJobList( QStringList );

    //! request clear queue
    void _requestClearQueue( QString );

    //! active frame
    void _setActiveFrame( JobManagerFrame& frame );

    //! create a new frame
    JobManagerFrame& _newFrame( void );

    //! get list of all active users
    UserSet _users( void ) const;

    //! true if default user is set
    bool _hasDefaultUser( void ) const
    { return !defaultUser_.isEmpty(); }

    //! true if a master job can be resubmitted
    bool _canResubmit( const Job& ) const;

    //! get commands associated to master job resubmission
    AlienCommand::List _masterJobResubmissionCommands( const Job& ) const;

    private Q_SLOTS:

    //! update configuration
    void _updateConfiguration( void );

    //! save configuration
    void _saveConfiguration( void );

    private:

    //! summary
    void _installActions( void );

    //! pointer to top interface
    AlienInterface* interface_;

    //! command model
    AlienCommandModel model_;

    //! log dialog
    LogDialog* logDialog_;

    //! toolbar
    CustomToolBar* userToolBar_;

    //! toolbar
    CustomToolBar* toolbar_;

    //! user
    UserSelectionFrame* userSelectionFrame_;

    //! status bar
    ProgressStatusBar* statusBar_;

    //! tab widget
    AnimatedTabWidget* tabWidget_;

    //! active frame
    JobManagerFrame* activeFrame_;

    //! busy widget
    BusyWidget* busyWidget_;

    //! refresh rate
    int refreshRate_;

    //! timer object for delayed tab change update
    QBasicTimer delayTimer_;

    //! timer object for scheduled update
    QBasicTimer timer_;

    //! default user
    QString defaultUser_;

    //! true if auto resubmission is requested
    bool autoResubmissionEnabled_;

    //! maximum resubmition
    int maxResubmissions_;

    //!@name actions
    //@{

    //! connection
    QAction* connectAction_;

    //! command list
    QAction* commandListAction_;

    //! log action
    QAction* logAction_;

    //! log action
    QAction* toggleLogAction_;

    //! hide killed jobs
    QAction* hideKilledJobsAction_;

    //! pause
    QAction* pauseAction_;

    //! default user action
    QAction* defaultUserAction_;

    //! user list action
    QAction* userListAction_;

    //! update
    QAction* updateAction_;

    //! retrace
    QAction* traceAction_;

    //! kill action
    QAction* killAction_;

    //! resubmit
    QAction* resubmitAction_;

    //! toggle summary frame
    QAction* toggleSummaryAction_;

    //! add frame
    QAction* addFrameAction_;

    //! remove frame
    QAction* removeFrameAction_;

    //! reset time stamps
    QAction* resetTimeStampsAction_;

    //! clear
    QAction* clearAction_;

    //! abort
    QAction* abortAction_;

    //! close job dialogs
    QAction* closeJobDialogsAction_;

    //@}

    //! true if connection was valid at least once
    bool wasConnected_;

};

#endif
